@@ -206,7 +206,9 @@ br_status array_rewriter::mk_store_core(unsigned num_args, expr * const * args,
 bool array_rewriter::squash_store(unsigned n, expr* const* args, expr_ref& result) {
     ptr_buffer<expr> parents, sargs;
     expr* a = args[0];
-    while (m_util.is_store(a)) {
+    unsigned rounds = 0;
+    while (m_util.is_store(a) && rounds < 10) {
+        ++rounds;
         lbool r = compare_args(n - 2, args + 1, to_app(a)->get_args() + 1);
         switch (r) {
         case l_undef:
@@ -264,15 +266,7 @@ br_status array_rewriter::mk_select_same_store(unsigned num_args, expr * const *
                 continue;
 
             case l_undef:
-                // check if loading from subsequent arrays yields the same value
-                if (first) {
-                    result = to_app(arg0)->get_arg(num_args);
-                    first  = false;
-                }
-                else if (result != to_app(arg0)->get_arg(num_args))
-                    return BR_FAILED;
-                arg0 = to_app(arg0)->get_arg(0);
-                continue;
+                return BR_FAILED;
             }
         }
 
