@@ -17,6 +17,7 @@ Notes:
 
 --*/
 #include "params/rewriter_params.hpp"
+#include "params/poly_rewriter_params.hpp"
 #include "ast/rewriter/th_rewriter.h"
 #include "ast/rewriter/bool_rewriter.h"
 #include "ast/rewriter/arith_rewriter.h"
@@ -58,6 +59,7 @@ struct th_rewriter_cfg : public default_rewriter_cfg {
     bv_util             m_bv_util;
     der                 m_der;
     expr_safe_replace   m_rep;
+    unused_vars_eliminator m_elim_unused_vars;
     expr_ref_vector     m_pinned;
       // substitution support
     expr_dependency_ref m_used_dependencies; // set of dependencies of used substitutions
@@ -83,7 +85,8 @@ struct th_rewriter_cfg : public default_rewriter_cfg {
 
     void updt_local_params(params_ref const & _p) {
         rewriter_params p(_p);
-        m_flat           = true;
+        poly_rewriter_params pp(_p);
+        m_flat           = pp.flat();
         m_max_memory     = megabytes_to_bytes(p.max_memory());
         m_max_steps      = p.max_steps();
         m_pull_cheap_ite = p.pull_cheap_ite();
@@ -595,9 +598,9 @@ struct th_rewriter_cfg : public default_rewriter_cfg {
         }
         app_ref tmp(m());
         tmp = m().mk_app(f, num, args);
-        m().trace_stream() << "[inst-discovered] theory-solving " << static_cast<void *>(nullptr) << " " << m().get_family_name(fid) << "# ; #" << tmp->get_id() << "\n";
+        m().trace_stream() << "[inst-discovered] theory-solving 0x0 " << m().get_family_name(fid) << "# ; #" << tmp->get_id() << "\n";
         tmp = m().mk_eq(tmp, result);
-        m().trace_stream() << "[instance] " << static_cast<void *>(nullptr) << " #" << tmp->get_id() << "\n";
+        m().trace_stream() << "[instance] 0x0 #" << tmp->get_id() << "\n";
 
         // Make sure that both the result term and equality were newly introduced.
         if (tmp->get_ref_count() == 1) {
@@ -683,9 +686,18 @@ struct th_rewriter_cfg : public default_rewriter_cfg {
             st = m_seq_rw.mk_eq_core(a, b, result);
         if (st != BR_FAILED)
             return st;
+        st = extended_bv_eq(a, b, result);
+        if (st != BR_FAILED)
+            return st;        
         return apply_tamagotchi(a, b, result);        
     }
 
+    br_status extended_bv_eq(expr* a, expr* b, expr_ref& result) {
+        if (m_bv_util.is_ubv2int(a) || m_bv_util.is_ubv2int(b))
+            return m_bv_rw.mk_eq_bv2int(a, b, result);
+        return BR_FAILED;        
+    }
+
     expr_ref mk_eq(expr* a, expr* b) {
         expr_ref result(m());
         br_status st = reduce_eq(a, b, result);
@@ -818,8 +830,7 @@ struct th_rewriter_cfg : public default_rewriter_cfg {
             }
         }
         SASSERT(old_q->get_sort() == q1->get_sort());
-        result = elim_unused_vars(m(), q1, params_ref());
-
+        result = m_elim_unused_vars(q1);        
 
 
         result_pr = nullptr;
@@ -876,6 +887,7 @@ struct th_rewriter_cfg : public default_rewriter_cfg {
         m_bv_util(m),
         m_der(m),
         m_rep(m),
+        m_elim_unused_vars(m, params_ref()),
         m_pinned(m),
         m_used_dependencies(m) {
         updt_local_params(p);
