@@ -4,6 +4,7 @@
 */
 #include "math/lp/lar_solver.h"
 #include "smt/params/smt_params_helper.hpp"
+#include "lar_solver.h"
 
 
 namespace lp {
@@ -44,7 +45,7 @@ namespace lp {
     bool lar_solver::sizes_are_correct() const {
         lp_assert(A_r().column_count() == m_mpq_lar_core_solver.m_r_solver.m_column_types.size());
         lp_assert(A_r().column_count() == m_mpq_lar_core_solver.m_r_solver.m_costs.size());
-        lp_assert(A_r().column_count() == m_mpq_lar_core_solver.m_r_x.size());
+        lp_assert(A_r().column_count() == m_mpq_lar_core_solver.r_x().size());
         return true;
     }
 
@@ -64,8 +65,8 @@ namespace lp {
     }
 
     std::ostream& lar_solver::print_values(std::ostream& out) const {
-        for (unsigned i = 0; i < m_mpq_lar_core_solver.m_r_x.size(); i++) {
-            const numeric_pair<mpq>& rp = m_mpq_lar_core_solver.m_r_x[i];
+        for (unsigned i = 0; i < m_mpq_lar_core_solver.r_x().size(); i++) {
+            const numeric_pair<mpq>& rp = m_mpq_lar_core_solver.r_x(i);
             out << this->get_variable_name(i) << " -> " << rp << "\n";
         }
         return out;
@@ -263,7 +264,7 @@ namespace lp {
             return false;
         }
         else {
-            term_max = term.apply(m_mpq_lar_core_solver.m_r_x);
+            term_max = term.apply(m_mpq_lar_core_solver.r_x());
             return true;
         }
     }
@@ -417,7 +418,7 @@ namespace lp {
 
     bool lar_solver::move_non_basic_column_to_bounds(unsigned j) {
         auto& lcs = m_mpq_lar_core_solver;
-        auto& val = lcs.m_r_x[j];
+        auto& val = lcs.r_x(j);
         switch (lcs.m_column_types()[j]) {
         case column_type::boxed: {
             const auto& l = lcs.m_r_lower_bounds()[j];
@@ -457,7 +458,7 @@ namespace lp {
 
     void lar_solver::set_value_for_nbasic_column(unsigned j, const impq& new_val) {
         lp_assert(!is_base(j));
-        auto& x = m_mpq_lar_core_solver.m_r_x[j];
+        auto& x = m_mpq_lar_core_solver.r_x(j);
         auto delta = new_val - x;
         x = new_val;
         TRACE("lar_solver_feas", tout << "setting " << j << " to "
@@ -504,7 +505,7 @@ namespace lp {
         if (column_has_term(j)) {
             return * m_columns[j].term();
         }
-        if (j < m_mpq_lar_core_solver.m_r_x.size()) {
+        if (j < m_mpq_lar_core_solver.r_x().size()) {
             lar_term r;
             r.add_monomial(one_of_type<mpq>(), j);
             return r;
@@ -518,41 +519,44 @@ namespace lp {
         SASSERT(m_mpq_lar_core_solver.m_r_solver.calc_current_x_is_feasible_include_non_basis());
         lar_term term = get_term_to_maximize(j);
         if (term.is_empty()) return lp_status::UNBOUNDED;
-        impq prev_value = term.apply(m_mpq_lar_core_solver.m_r_x);
-        auto backup = m_mpq_lar_core_solver.m_r_x;
+        m_mpq_lar_core_solver.backup_x();
+        impq prev_value = term.apply(m_mpq_lar_core_solver.r_x());
+        auto restore = [&]() {
+            m_mpq_lar_core_solver.restore_x();
+        };
         if (!maximize_term_on_feasible_r_solver(term, term_max, nullptr)) {
-            m_mpq_lar_core_solver.m_r_x = backup;
+            restore();
             return lp_status::UNBOUNDED;
         }
 
         impq opt_val = term_max;
 
         bool change = false;
-        for (unsigned j = 0; j < m_mpq_lar_core_solver.m_r_x.size(); j++) {
+        for (unsigned j = 0; j < m_mpq_lar_core_solver.r_x().size(); j++) {
             if (!column_is_int(j))
                 continue;
             if (column_value_is_integer(j))
                 continue;
             if (m_int_solver->is_base(j)) {
                 if (!remove_from_basis(j)) { // consider a special version of remove_from_basis that would not remove inf_int columns
-                    m_mpq_lar_core_solver.m_r_x = backup;
+                    restore();
                     term_max = prev_value;
                     return lp_status::FEASIBLE; // it should not happen
                 }
             }
             if (!column_value_is_integer(j)) {
                 term_max = prev_value;
-                m_mpq_lar_core_solver.m_r_x = backup;
+                restore();
                 return lp_status::FEASIBLE;
             }
             change = true;
         }
         if (change) {
-            term_max = term.apply(m_mpq_lar_core_solver.m_r_x);
+            term_max = term.apply(m_mpq_lar_core_solver.r_x());
         }
         if (term_max < prev_value) {
             term_max = prev_value;
-            m_mpq_lar_core_solver.m_r_x = backup;
+            restore();
         }
         TRACE("lar_solver", print_values(tout););
         if (term_max == opt_val) {
@@ -570,8 +574,6 @@ namespace lp {
         A_r().pop(k);
     }
 
-
-
     void lar_solver::set_upper_bound_witness(lpvar j, u_dependency* dep) {
         m_trail.push(vector_value_trail(m_columns, j));
         m_columns[j].upper_bound_witness() = dep;
@@ -617,6 +619,159 @@ namespace lp {
             m_touched_rows.insert(rid);
     }
 
+    void lar_solver::check_fixed(unsigned j) {
+        if (column_is_fixed(j))
+            return;
+
+        auto explain = [&](constraint_index ci, lp::explanation const& exp) {
+            u_dependency* d = nullptr;
+            for (auto const& e : exp)
+                if (e.ci() != ci) 
+                    d = join_deps(d, m_dependencies.mk_leaf(e.ci()));                
+            return d;
+        };
+
+        if (!column_has_lower_bound(j) || column_lower_bound(j) != get_value(j)) {
+            push();
+            mpq val = get_value(j);
+            auto ci = add_var_bound(j, lconstraint_kind::LT, val);
+            auto r = solve();
+            lp::explanation exp;
+            if (r == lp_status::INFEASIBLE) 
+                get_infeasibility_explanation(exp);            
+            pop();
+            if (r == lp_status::INFEASIBLE) {
+                auto d = explain(ci, exp);
+                update_column_type_and_bound(j, lconstraint_kind::GE, val, d);
+            }
+            solve();
+        }
+
+        if (!column_has_upper_bound(j) || column_upper_bound(j) != get_value(j)) {
+            push();
+            auto val = get_value(j);
+            auto ci = add_var_bound(j, lconstraint_kind::GT, val);
+            auto r = solve();
+            lp::explanation exp;
+            if (r == lp_status::INFEASIBLE) 
+                get_infeasibility_explanation(exp);
+            pop();
+            if (r == lp_status::INFEASIBLE) {
+                auto d = explain(ci, exp);
+                update_column_type_and_bound(j, lconstraint_kind::LE, val, d);
+            }
+            solve();
+        }
+    }
+
+    void lar_solver::solve_for(unsigned_vector const& js, vector<solution>& sols) {
+        uint_set tabu, fixed_checked;
+        for (auto j : js)
+            solve_for(j, tabu, sols);
+        solve();  // clear updated columns.
+        auto check = [&](unsigned j) {
+            if (fixed_checked.contains(j))
+                return;
+            fixed_checked.insert(j);
+            check_fixed(j);
+        };
+        for (auto const& [j, t] : sols) {
+            check(j);
+            for (auto const& v : t) 
+                check(v.j());            
+        }
+    }
+
+    void lar_solver::solve_for(unsigned j, uint_set& tabu, vector<solution>& sols) {
+        if (tabu.contains(j))
+            return;
+        tabu.insert(j);
+        IF_VERBOSE(10, verbose_stream() << "solve for " << j << " base " << is_base(j) << " " << column_is_fixed(j) << "\n");
+        TRACE("arith", tout << "solve for " << j << " base " << is_base(j) << " " << column_is_fixed(j) << "\n");
+        if (column_is_fixed(j)) 
+            return;
+        
+        if (!is_base(j)) {
+            for (const auto& c : A_r().m_columns[j]) {
+                lpvar basic_in_row = r_basis()[c.var()];
+                if (tabu.contains(basic_in_row))
+                    continue;
+                pivot(j, basic_in_row);
+                break;
+            }
+        }
+        if (!is_base(j)) 
+            return;
+        
+        lar_term t;
+        //auto const& col = m_columns[j];
+        auto const& r = basic2row(j);
+        for (auto const& c : r) {
+            if (c.var() != j)       
+                t.add_monomial(-c.coeff(), c.var());            
+        }            
+        for (auto const& v : t) 
+            solve_for(v.j(), tabu, sols); 
+        lp::impq lo, hi;
+        bool lo_valid = true, hi_valid = true;
+        for (auto const& v : t) {
+            if (v.coeff().is_pos()) {
+                if (lo_valid && column_has_lower_bound(v.j()))
+                    lo += column_lower_bound(v.j()) * v.coeff();
+                else
+                    lo_valid = false;
+                if (hi_valid && column_has_upper_bound(v.j()))
+                    hi += column_upper_bound(v.j()) * v.coeff();
+                else
+                    hi_valid = false;
+            }
+            else {
+                if (lo_valid && column_has_upper_bound(v.j()))
+                    lo += column_upper_bound(v.j()) * v.coeff();
+                else
+                    lo_valid = false;
+                if (hi_valid && column_has_lower_bound(v.j()))
+                    hi += column_lower_bound(v.j()) * v.coeff();
+                else
+                    hi_valid = false;
+            }
+        }        
+
+        if (lo_valid && (!column_has_lower_bound(j) || lo > column_lower_bound(j).x)) {
+            u_dependency* dep = nullptr;
+            for (auto const& v : t) {
+                if (v.coeff().is_pos())
+                    dep = join_deps(dep, m_columns[v.j()].lower_bound_witness());
+                else
+                    dep = join_deps(dep, m_columns[v.j()].upper_bound_witness());
+            }
+            update_column_type_and_bound(j, lo.y == 0 ? lconstraint_kind::GE : lconstraint_kind::GT, lo.x, dep);
+        }
+
+        if (hi_valid && (!column_has_upper_bound(j) || hi < column_upper_bound(j).x)) {
+            u_dependency* dep = nullptr;
+            for (auto const& v : t) {
+                if (v.coeff().is_pos())
+                    dep = join_deps(dep, m_columns[v.j()].upper_bound_witness());
+                else
+                    dep = join_deps(dep, m_columns[v.j()].lower_bound_witness());
+            }
+            update_column_type_and_bound(j, hi.y == 0 ? lconstraint_kind::LE : lconstraint_kind::LT, hi.x, dep);
+        }
+
+        TRACE("arith", print_term(t, tout << "j" << j << " := ") << "\n");
+        if (!column_is_fixed(j))             
+            sols.push_back({j, t});
+    }
+
+
+    void lar_solver::explain_fixed_column(unsigned j, explanation& ex) {
+        SASSERT(column_is_fixed(j));
+        auto* deps = get_bound_constraint_witnesses_for_column(j);
+        for (auto ci : flatten(deps))
+            ex.push_back(ci);
+    }
+
     void lar_solver::remove_fixed_vars_from_base() {
         // this will allow to disable and restore the tracking of the touched rows
         flet<indexed_uint_set*> f(m_mpq_lar_core_solver.m_r_solver.m_touched_rows, nullptr);
@@ -671,7 +826,7 @@ namespace lp {
     bool lar_solver::row_is_correct(unsigned i) const {
         numeric_pair<mpq> r = zero_of_type<numeric_pair<mpq>>();
         for (const auto& c : A_r().m_rows[i]) {
-            r += c.coeff() * m_mpq_lar_core_solver.m_r_x[c.var()];
+            r += c.coeff() * m_mpq_lar_core_solver.r_x(c.var());
         }
         CTRACE("lar_solver", !is_zero(r), tout << "row = " << i << ", j = " << m_mpq_lar_core_solver.m_r_basis[i] << "\n";
                print_row(A_r().m_rows[i], tout);  tout << " = " << r << "\n");
@@ -780,7 +935,7 @@ namespace lp {
         unsigned bj = m_mpq_lar_core_solver.m_r_solver.m_basis[i];
         for (const auto& c : A_r().m_rows[i]) {
             if (c.var() == bj) continue;
-            const auto& x = m_mpq_lar_core_solver.m_r_x[c.var()];
+            const auto& x = m_mpq_lar_core_solver.r_x(c.var());
             if (!is_zero(x))
                 r -= c.coeff() * x;
         }
@@ -910,16 +1065,22 @@ namespace lp {
         }
         return ret;
     }
-
-    bool lar_solver::has_lower_bound(lpvar var, u_dependency*& ci, mpq& value, bool& is_strict) const {
+    bool lar_solver::has_bound_of_type(lpvar var, u_dependency*& ci, mpq& value, bool& is_strict, bool is_upper) const {
+        if (is_upper) {
+            return has_upper_bound(var, ci, value, is_strict);
+        } else {
+            return has_lower_bound(var, ci, value, is_strict);
+        }
+    }
+    bool lar_solver::has_lower_bound(lpvar var, u_dependency*& dep, mpq& value, bool& is_strict) const {
 
         if (var >= m_columns.size()) {
             // TBD: bounds on terms could also be used, caller may have to track these.
             return false;
         }
         const column& ul = m_columns[var];
-        ci = ul.lower_bound_witness();
-        if (ci != nullptr) {
+        dep = ul.lower_bound_witness();
+        if (dep != nullptr) {
             auto& p = m_mpq_lar_core_solver.m_r_lower_bounds()[var];
             value = p.x;
             is_strict = p.y.is_pos();
@@ -930,15 +1091,15 @@ namespace lp {
         }
     }
 
-    bool lar_solver::has_upper_bound(lpvar var, u_dependency*& ci, mpq& value, bool& is_strict) const {
+    bool lar_solver::has_upper_bound(lpvar var, u_dependency*& dep, mpq& value, bool& is_strict) const {
 
         if (var >= m_columns.size()) {
             // TBD: bounds on terms could also be used, caller may have to track these.
             return false;
         }
         const column& ul = m_columns[var];
-        ci = ul.upper_bound_witness();
-        if (ci != nullptr) {
+        dep = ul.upper_bound_witness();
+        if (dep != nullptr) {
             auto& p = m_mpq_lar_core_solver.m_r_upper_bounds()[var];
             value = p.x;
             is_strict = p.y.is_neg();
@@ -1012,7 +1173,7 @@ namespace lp {
         if (!init_model())
             return;
 
-        unsigned n = m_mpq_lar_core_solver.m_r_x.size();
+        unsigned n = m_mpq_lar_core_solver.r_x().size();
 
         for (unsigned j = 0; j < n; j++) 
             variable_values[j] = get_value(j);
@@ -1022,11 +1183,15 @@ namespace lp {
     }
 
     bool lar_solver::init_model() const {
+        auto& rslv = m_mpq_lar_core_solver.m_r_solver;
+        lp_assert(A_r().column_count() == rslv.m_costs.size());
+        lp_assert(A_r().column_count() == m_mpq_lar_core_solver.r_x().size());
+        lp_assert(A_r().column_count() == rslv.m_d.size());
         CTRACE("lar_solver_model",!m_columns_with_changed_bounds.empty(), tout << "non-empty changed bounds\n");
         TRACE("lar_solver_model", tout << get_status() << "\n");
         auto status = get_status();
         SASSERT((status != lp_status::OPTIMAL && status != lp_status::FEASIBLE) 
-            || m_mpq_lar_core_solver.m_r_solver.calc_current_x_is_feasible_include_non_basis());
+            || rslv.calc_current_x_is_feasible_include_non_basis());
         if (status != lp_status::OPTIMAL && status != lp_status::FEASIBLE)
             return false;
         if (!m_columns_with_changed_bounds.empty())
@@ -1034,12 +1199,12 @@ namespace lp {
 
         m_delta = m_mpq_lar_core_solver.find_delta_for_strict_bounds(mpq(1));
         unsigned j;
-        unsigned n = m_mpq_lar_core_solver.m_r_x.size();
+        unsigned n = m_mpq_lar_core_solver.r_x().size();
         do {
             m_set_of_different_pairs.clear();
             m_set_of_different_singles.clear();
             for (j = 0; j < n; j++) {
-                const numeric_pair<mpq>& rp = m_mpq_lar_core_solver.m_r_x[j];
+                const numeric_pair<mpq>& rp = m_mpq_lar_core_solver.r_x(j);
                 mpq x = rp.x + m_delta * rp.y;
                 m_set_of_different_pairs.insert(rp);
                 m_set_of_different_singles.insert(x);
@@ -1055,8 +1220,8 @@ namespace lp {
 
     void lar_solver::get_model_do_not_care_about_diff_vars(std::unordered_map<lpvar, mpq>& variable_values) const {
         mpq delta = m_mpq_lar_core_solver.find_delta_for_strict_bounds(mpq(1));
-        for (unsigned i = 0; i < m_mpq_lar_core_solver.m_r_x.size(); i++) {
-            const impq& rp = m_mpq_lar_core_solver.m_r_x[i];
+        for (unsigned i = 0; i < m_mpq_lar_core_solver.r_x().size(); i++) {
+            const impq& rp = m_mpq_lar_core_solver.r_x(i);
             variable_values[i] = rp.x + delta * rp.y;
         }
     }
@@ -1071,7 +1236,7 @@ namespace lp {
     void lar_solver::get_rid_of_inf_eps() {
         bool y_is_zero = true;
         for (unsigned j = 0; j < number_of_vars(); j++) {
-            if (!m_mpq_lar_core_solver.m_r_x[j].y.is_zero()) {
+            if (!m_mpq_lar_core_solver.r_x(j).y.is_zero()) {
                 y_is_zero = false;
                 break;
             }
@@ -1080,7 +1245,7 @@ namespace lp {
             return;
         mpq delta = m_mpq_lar_core_solver.find_delta_for_strict_bounds(mpq(1));
         for (unsigned j = 0; j < number_of_vars(); j++) {
-            auto& v = m_mpq_lar_core_solver.m_r_x[j];
+            auto& v = m_mpq_lar_core_solver.r_x(j);
             if (!v.y.is_zero()) {
                 v = impq(v.x + delta * v.y);
                 TRACE("lar_solver_feas", tout << "x[" << j << "] = " << v << "\n";);
@@ -1214,7 +1379,7 @@ namespace lp {
         lp_assert(A_r().row_count() == i + 1 && A_r().column_count() == j + 1);
         auto& last_column = A_r().m_columns[j];
         int non_zero_column_cell_index = -1;
-        for (unsigned k = last_column.size(); k-- > 0;) {
+        for (unsigned k = static_cast<unsigned>(last_column.size()); k-- > 0;) {
             auto& cc = last_column[k];
             if (cc.var() == i)
                 return;
@@ -1238,7 +1403,7 @@ namespace lp {
         auto& last_row = A_r().m_rows[i];
         mpq& cost_j = m_mpq_lar_core_solver.m_r_solver.m_costs[j];
         bool cost_is_nz = !is_zero(cost_j);
-        for (unsigned k = last_row.size(); k-- > 0;) {
+        for (unsigned k = static_cast<unsigned>(last_row.size()); k-- > 0;) {
             auto& rc = last_row[k];
             if (cost_is_nz) {
                 m_mpq_lar_core_solver.m_r_solver.m_d[rc.var()] += cost_j * rc.coeff();
@@ -1290,7 +1455,7 @@ namespace lp {
     void lar_solver::remove_last_column_from_tableau() {
         auto& rslv = m_mpq_lar_core_solver.m_r_solver;
         unsigned j = A_r().column_count() - 1;
-        lp_assert(A_r().column_count() == m_mpq_lar_core_solver.m_r_solver.m_costs.size());
+        lp_assert(A_r().column_count() == rslv.m_costs.size());
         if (column_represents_row_in_tableau(j)) {
             remove_last_row_and_column_from_tableau(j);
             if (rslv.m_basis_heading[j] < 0)
@@ -1299,13 +1464,15 @@ namespace lp {
         else {
             remove_last_column_from_A();
         }
-        rslv.m_x.pop_back();
+        m_mpq_lar_core_solver.resize_x(A_r().column_count());
         rslv.m_d.pop_back();
         rslv.m_costs.pop_back();
 
         remove_last_column_from_basis_tableau(j);
         lp_assert(m_mpq_lar_core_solver.r_basis_is_OK());
-        lp_assert(A_r().column_count() == m_mpq_lar_core_solver.m_r_solver.m_costs.size());
+        lp_assert(A_r().column_count() == rslv.m_costs.size());
+        lp_assert(A_r().column_count() == m_mpq_lar_core_solver.r_x().size());
+        lp_assert(A_r().column_count() == rslv.m_d.size());
     }
 
 
@@ -1402,7 +1569,7 @@ namespace lp {
                 if (s.m_need_register_terms)
                     s.deregister_normalized_term(*col.term());
                 delete col.term();
-                s.m_terms.pop_back();    
+                s.m_terms.pop_back();
             }
             s.remove_last_column_from_tableau();            
             s.m_columns.pop_back();
@@ -1421,9 +1588,9 @@ namespace lp {
             return local_j;
         lp_assert(m_columns.size() == A_r().column_count());
         local_j = A_r().column_count();
-        m_columns.push_back(column(false, nullptr)); // false - not associated with a row, nullptr for term
+        m_columns.push_back(column());
         m_trail.push(undo_add_column(*this));
-        while (m_usage_in_terms.size() <= ext_j) 
+        while (m_usage_in_terms.size() <= local_j) 
             m_usage_in_terms.push_back(0);
         add_non_basic_var_to_core_fields(ext_j, is_int);
         lp_assert(sizes_are_correct());
@@ -1442,7 +1609,7 @@ namespace lp {
     bool lar_solver::external_is_used(unsigned v) const {
         return m_var_register.external_is_used(v);
     }
-
+    
     void lar_solver::add_non_basic_var_to_core_fields(unsigned ext_j, bool is_int) {
         register_new_external_var(ext_j, is_int);
         m_mpq_lar_core_solver.m_column_types.push_back(column_type::free_column);
@@ -1453,14 +1620,15 @@ namespace lp {
         unsigned j = A_r().column_count();
         TRACE("add_var", tout << "j = " << j << std::endl;);
         A_r().add_column();
-        lp_assert(m_mpq_lar_core_solver.m_r_x.size() == j);
+        lp_assert(m_mpq_lar_core_solver.r_x().size() == j);
         //        lp_assert(m_mpq_lar_core_solver.m_r_lower_bounds.size() == j && m_mpq_lar_core_solver.m_r_upper_bounds.size() == j);  // restore later
-        m_mpq_lar_core_solver.m_r_x.resize(j + 1);
+        m_mpq_lar_core_solver.resize_x(j + 1);
+        auto& rslv = m_mpq_lar_core_solver.m_r_solver;
         m_mpq_lar_core_solver.m_r_lower_bounds.increase_size_by_one();
         m_mpq_lar_core_solver.m_r_upper_bounds.increase_size_by_one();
-        m_mpq_lar_core_solver.m_r_solver.inf_heap_increase_size_by_one();
-        m_mpq_lar_core_solver.m_r_solver.m_costs.resize(j + 1);
-        m_mpq_lar_core_solver.m_r_solver.m_d.resize(j + 1);
+        rslv.inf_heap_increase_size_by_one();
+        rslv.m_costs.resize(j + 1);
+        rslv.m_d.resize(j + 1);
         lp_assert(m_mpq_lar_core_solver.m_r_heading.size() == j); // as A().column_count() on the entry to the method
         if (register_in_basis) {
             A_r().add_row();
@@ -1531,7 +1699,7 @@ namespace lp {
         SASSERT(all_vars_are_registered(coeffs));
         lar_term* t = new lar_term(coeffs);
         subst_known_terms(t);
-        SASSERT(t->is_empty() == false);
+        SASSERT (!t->is_empty());
         m_terms.push_back(t);
         lpvar ret = A_r().column_count();
         add_row_from_term_no_constraint(t, ext_i);
@@ -1539,6 +1707,8 @@ namespace lp {
         lp_assert(m_var_register.size() == A_r().column_count());
         if (m_need_register_terms) 
             register_normalized_term(*t, A_r().column_count() - 1);
+        if (m_add_term_callback)
+            m_add_term_callback(t);    
         return ret;
     }
 
@@ -1548,8 +1718,8 @@ namespace lp {
         // j will be a new variable
         unsigned j = A_r().column_count();
         SASSERT(ext_index == null_lpvar || external_to_local(ext_index) == j);
-        column ul(true, term); // true - to mark this column as associated_with_row
-        term->j() = j; // point from the term to the column
+        column ul(term);
+        term->set_j(j); // point from the term to the column
         m_columns.push_back(ul);
         m_trail.push(undo_add_column(*this));
         add_basic_var_to_core_fields();
@@ -1657,10 +1827,12 @@ namespace lp {
         switch (k) {
         case LT:
             k = LE;
+            Z3_fallthrough;
         case LE:
             return floor(bound);
         case GT:
             k = GE;
+            Z3_fallthrough;
         case GE:
             return ceil(bound);
         case EQ:
@@ -1798,7 +1970,8 @@ namespace lp {
         // SASSERT(validate_bound(j, kind, right_side, dep));
         TRACE(
             "lar_solver_feas",
-            tout << "j" << j << " " << lconstraint_kind_string(kind) << " " << right_side << std::endl;            
+            tout << "j" << j << " " << lconstraint_kind_string(kind) << " " << right_side << std::endl;   
+            print_column_info(j, tout) << "\n";
             if (dep) {
                 tout << "dep:\n";
                 auto cs = flatten(dep);
@@ -1807,15 +1980,23 @@ namespace lp {
                     tout << std::endl;
                 }
             });
+        bool was_fixed = column_is_fixed(j);
         mpq rs = adjust_bound_for_int(j, kind, right_side);
         if (column_has_upper_bound(j))
             update_column_type_and_bound_with_ub(j, kind, rs, dep);
         else
             update_column_type_and_bound_with_no_ub(j, kind, rs, dep);
 
-        if (is_base(j) && column_is_fixed(j))
+        if (!was_fixed && column_is_fixed(j) && m_fixed_var_eh)
+            m_fixed_var_eh(j);
+
+        if (is_base(j) && column_is_fixed(j)) 
             m_fixed_base_var_set.insert(j);
+        
+
         TRACE("lar_solver_feas", tout << "j = " << j << " became " << (this->column_is_feasible(j) ? "feas" : "non-feas") << ", and " << (this->column_is_bounded(j) ? "bounded" : "non-bounded") << std::endl;);
+        if (m_update_column_bound_callback)
+            m_update_column_bound_callback(j);
     }
 
     void lar_solver::insert_to_columns_with_changed_bounds(unsigned j) {
@@ -1840,13 +2021,13 @@ namespace lp {
         return m_constraints.add_term_constraint(j, m_columns[j].term(), kind, rs);
     }
 
-    struct scoped_backup {
+    struct lar_solver::scoped_backup {
         lar_solver& m_s;
         scoped_backup(lar_solver& s) : m_s(s) {
-            m_s.backup_x(); 
+            m_s.m_mpq_lar_core_solver.backup_x();
         }
         ~scoped_backup() {
-            m_s.restore_x();
+            m_s.m_mpq_lar_core_solver.restore_x();
         }
     };
    
@@ -1879,6 +2060,7 @@ namespace lp {
         switch (kind) {
             case LT:
                 y_of_bound = -1;
+                Z3_fallthrough;
             case LE: {
                 auto up = numeric_pair<mpq>(right_side, y_of_bound);
                 if (up < m_mpq_lar_core_solver.m_r_lower_bounds[j]) {
@@ -1894,6 +2076,7 @@ namespace lp {
             }
             case GT:
                 y_of_bound = 1;
+                Z3_fallthrough;
             case GE: {
                 auto low = numeric_pair<mpq>(right_side, y_of_bound);
                 if (low > m_mpq_lar_core_solver.m_r_upper_bounds[j]) {
@@ -1930,9 +2113,10 @@ namespace lp {
             default:
                 UNREACHABLE();
         }
-        if (m_mpq_lar_core_solver.m_r_upper_bounds[j] == m_mpq_lar_core_solver.m_r_lower_bounds[j]) {
+        numeric_pair<mpq> const& lo = m_mpq_lar_core_solver.m_r_lower_bounds[j];
+        numeric_pair<mpq> const& hi = m_mpq_lar_core_solver.m_r_upper_bounds[j];
+        if (lo == hi)
             m_mpq_lar_core_solver.m_column_types[j] = column_type::fixed;
-        }
     }
     
     void lar_solver::update_bound_with_no_ub_lb(lpvar j, lconstraint_kind kind, const mpq& right_side, u_dependency* dep) {
@@ -1943,6 +2127,7 @@ namespace lp {
         switch (kind) {
             case LT:
                 y_of_bound = -1;
+                Z3_fallthrough;
             case LE: {
                 auto up = numeric_pair<mpq>(right_side, y_of_bound);
                 if (up < m_mpq_lar_core_solver.m_r_lower_bounds[j]) {
@@ -1995,6 +2180,7 @@ namespace lp {
         switch (kind) {
         case LT:
             y_of_bound = -1;
+            Z3_fallthrough;
         case LE:
         {
             auto up = numeric_pair<mpq>(right_side, y_of_bound);
@@ -2006,6 +2192,7 @@ namespace lp {
         break;
         case GT:
             y_of_bound = 1;
+            Z3_fallthrough;
         case GE:
         {
             auto low = numeric_pair<mpq>(right_side, y_of_bound);
@@ -2048,6 +2235,7 @@ namespace lp {
         switch (kind) {
             case LT:
                 y_of_bound = -1;
+                Z3_fallthrough;
             case LE: {
                 auto up = numeric_pair<mpq>(right_side, y_of_bound);
                 m_mpq_lar_core_solver.m_r_upper_bounds[j] = up;
@@ -2056,6 +2244,7 @@ namespace lp {
             } break;
             case GT:
                 y_of_bound = 1;
+                Z3_fallthrough;
             case GE: {
                 auto low = numeric_pair<mpq>(right_side, y_of_bound);
                 m_mpq_lar_core_solver.m_r_lower_bounds[j] = low;
@@ -2081,6 +2270,24 @@ namespace lp {
     lpvar lar_solver::to_column(unsigned ext_j) const {
         return m_var_register.external_to_local(ext_j);
     }
+    
+    bool lar_solver::move_lpvar_to_value(lpvar j, mpq const& value) {
+        if (is_base(j))
+            return false;
+
+        impq ivalue(value);
+        auto& lcs = m_mpq_lar_core_solver;
+        auto& slv = m_mpq_lar_core_solver.m_r_solver;
+
+        if (slv.column_has_upper_bound(j) && lcs.m_r_upper_bounds()[j] < ivalue)
+            return false;
+        if (slv.column_has_lower_bound(j) && lcs.m_r_lower_bounds()[j] > ivalue)
+            return false;
+        
+        set_value_for_nbasic_column(j, ivalue);
+        return true;
+    }
+
 
     bool lar_solver::tighten_term_bounds_by_delta(lpvar j, const impq& delta) {
         SASSERT(column_has_term(j));
@@ -2114,7 +2321,7 @@ namespace lp {
         for (unsigned j = 0; j < column_count(); j++) {
             if (!column_is_int(j)) continue;
             if (column_has_term(j)) continue;
-            impq& v = m_mpq_lar_core_solver.m_r_x[j];
+            impq & v = m_mpq_lar_core_solver.r_x(j);
             if (v.is_int())
                 continue;
             TRACE("cube", m_int_solver->display_column(tout, j););
@@ -2151,7 +2358,7 @@ namespace lp {
                 }
             }
             if (need_to_fix) {
-                impq v = t->apply(m_mpq_lar_core_solver.m_r_x);
+                impq v = t->apply(m_mpq_lar_core_solver.r_x());
                 m_mpq_lar_core_solver.m_r_solver.update_x(j, v);
             }
         }
@@ -2163,7 +2370,7 @@ namespace lp {
     bool lar_solver::sum_first_coords(const lar_term& t, mpq& val) const {
         val = zero_of_type<mpq>();
         for (lar_term::ival c : t) {
-            const auto& x = m_mpq_lar_core_solver.m_r_x[c.j()];
+            const auto& x = m_mpq_lar_core_solver.r_x(c.j());
             if (!is_zero(x.y))
                 return false;
             val += x.x * c.coeff();
@@ -2266,12 +2473,22 @@ namespace lp {
         return false;
     }
 
+    bool lar_solver::are_equal(lpvar j, lpvar k) {
+        vector<std::pair<mpq, lpvar>> coeffs;        
+        coeffs.push_back(std::make_pair(mpq(1), j));
+        coeffs.push_back(std::make_pair(mpq(-1), k));
+        lar_term t(coeffs);
+        subst_known_terms(&t);
+        return t.is_empty();
+    }
+
     std::pair<constraint_index, constraint_index> lar_solver::add_equality(lpvar j, lpvar k) {
         vector<std::pair<mpq, lpvar>> coeffs;
         
         coeffs.push_back(std::make_pair(mpq(1), j));
         coeffs.push_back(std::make_pair(mpq(-1), k));
         unsigned ej = add_term(coeffs, UINT_MAX); // UINT_MAX is the external null var
+            
 
         if (get_column_value(j) != get_column_value(k))
             set_status(lp_status::UNKNOWN);
