@@ -42,7 +42,6 @@ Revision History:
 #include "sat/sat_cut_simplifier.h"
 #include "sat/sat_probing.h"
 #include "sat/sat_mus.h"
-#include "sat/sat_binspr.h"
 #include "sat/sat_drat.h"
 #include "sat/sat_parallel.h"
 #include "sat/sat_local_search.h"
@@ -114,7 +113,6 @@ namespace sat {
         probing                 m_probing;
         bool                    m_is_probing { false };
         mus                     m_mus;           // MUS for minimal core extraction
-        binspr                  m_binspr;
         bool                    m_inconsistent;
         bool                    m_searching;
         // A conflict is usually a single justification. That is, a justification
@@ -215,7 +213,6 @@ namespace sat {
         friend class cleaner;
         friend class asymm_branch;
         friend class big;
-        friend class binspr;
         friend class drat;
         friend class elim_eqs;
         friend class bcd;
