@@ -456,6 +456,7 @@ namespace smt {
             literal2expr(~consequent, n);
             fmls.push_back(std::move(n));
         }
+
         if (logic != symbol::null) out << "(set-logic " << logic << ")\n";
         visitor.collect(fmls);
         visitor.display_decls(out);
@@ -475,7 +476,7 @@ namespace smt {
 
     void context::display_lemma_as_smt_problem(std::ostream & out, unsigned num_antecedents, literal const * antecedents,
                                                unsigned num_eq_antecedents, enode_pair const * eq_antecedents,
-                                               literal consequent, symbol const& logic) const {
+                                               literal consequent, symbol const& logic, enode* x, enode* y) const {
         ast_pp_util visitor(m);
         expr_ref_vector fmls(m);
         visitor.collect(fmls);
@@ -490,6 +491,10 @@ namespace smt {
             n = m.mk_eq(p.first->get_expr(), p.second->get_expr());
             fmls.push_back(n);
         }
+        if (x && y) {
+            expr_ref eq(m.mk_eq(x->get_expr(), y->get_expr()), m);
+            fmls.push_back(m.mk_not(eq));
+        }
         if (consequent != false_literal) {
             literal2expr(~consequent, n);
             fmls.push_back(n);
@@ -712,14 +717,14 @@ namespace smt {
         };
         std::stringstream strm;
         strm << "(smt.stats " 
-             << std::setw(4) << m_stats.m_num_restarts << " "
-             << std::setw(6) << m_stats.m_num_conflicts << " "
-             << std::setw(6) << m_stats.m_num_decisions << " " 
-             << std::setw(6) << m_stats.m_num_propagations << " "
-             << std::setw(5) << (m_aux_clauses.size() + bin_clauses) << "/" << bin_clauses << "/" << num_units()
-             << std::setw(7) << m_lemmas.size(); if (bin_lemmas > 0) strm << "/" << bin_lemmas << " ";
-        strm << std::setw(5) << m_stats.m_num_simplifications << " "
-             << std::setw(4) << m_stats.m_num_del_clauses << " "
+             << std::setw(4) << m_stats.m_num_restarts << ' '
+             << std::setw(6) << m_stats.m_num_conflicts << ' '
+             << std::setw(6) << m_stats.m_num_decisions << ' '
+             << std::setw(6) << m_stats.m_num_propagations << ' '
+             << std::setw(5) << (m_aux_clauses.size() + bin_clauses) << '/' << bin_clauses << '/' << num_units() << ' '
+             << std::setw(7) << m_lemmas.size() << '/' << bin_lemmas << ' '
+             << std::setw(5) << m_stats.m_num_simplifications << ' '
+             << std::setw(4) << m_stats.m_num_del_clauses << ' '
              << std::setw(7) << mem_stat() << ")\n";
 
         std::string str = std::move(strm).str();
@@ -745,7 +750,7 @@ namespace smt {
             m_last_position_log = m_stats.m_num_restarts;
             // restarts       decisions      clauses    simplifications  memory
             //      conflicts       propagations    lemmas       deletions
-            int adjust[9] = { -3, -3, -3, -3, -3, -4, -4, -4, -1 };
+            const int adjust[9] = { -3, -3, -3, -3, -3, -4, -4, -4, -1 };
             char const* tag[9] = { ":restarts ", ":conflicts ", ":decisions ", ":propagations ", ":clauses/bin/units ", ":lemmas ", ":simplify ", ":deletions", ":memory" };
 
             std::stringstream l1, l2;
