@@ -337,7 +337,8 @@ namespace {
 
         expr* congruence_next(expr* e) override { return m_context.congruence_next(e); }
         expr* congruence_root(expr* e) override { return m_context.congruence_root(e); }
-        bool  solve_for(expr* e, expr_ref& term) override { return m_context.solve_for(e, term); }
+        expr_ref congruence_explain(expr* a, expr* b) override { return m_context.congruence_explain(a, b); }
+        void  solve_for(vector<solver::solution>& s) override { m_context.solve_for(s); }
 
 
         expr_ref_vector cube(expr_ref_vector& vars, unsigned cutoff) override {
